/** @file ccp_scenario.h
 *
 * Scenario for verifying the cache coherence protocol.
 *
 * @author Alexander Kamkin (kamkin@ispras.ru)
 */

#pragma once

#include <ts/scenario.hpp>

#include <ccp_model.h>

using namespace cpptesk::ts;
using namespace cpptesk::hw;
using namespace cpptesk::tracer;

namespace cpptesk {
namespace examples {
namespace ccp {

SCENARIO(CacheCoherenceScenario)
{
public:
    typedef IntAbcCtx Ctx;

    static const unsigned N = 8;

    CacheCoherenceScenario(): model(N)
    {
        setup("CacheCoherenceScenario",
            &CacheCoherenceScenario::init,
            &CacheCoherenceScenario::finish,
            &CacheCoherenceScenario::getState);

        SCENARIO_METHOD(CacheCoherenceScenario::read);
        SCENARIO_METHOD(CacheCoherenceScenario::write);
        SCENARIO_METHOD(CacheCoherenceScenario::replace);
        SCENARIO_METHOD(CacheCoherenceScenario::reset);
    }

    virtual ~CacheCoherenceScenario() {}

    virtual bool init(int argc, char ** argv);
    virtual void finish();
    virtual std::string getState();

    bool read   (Ctx &ctx);
    bool write  (Ctx &ctx);
    bool replace(Ctx &ctx);
    bool reset  (Ctx &ctx);

protected:
    CacheCoherenceModel model;
};

class VectorStateScenario: public CacheCoherenceScenario
{
public:
    VectorStateScenario(): CacheCoherenceScenario()
    {
        setup("VectorStateScenario",
            &CacheCoherenceScenario::init,
            &CacheCoherenceScenario::finish,
            &VectorStateScenario::getState);
    }

    virtual ~VectorStateScenario() {}

    virtual std::string getState() { return model.getState(); }
};

class MultiSetStateScenario: public CacheCoherenceScenario
{
public:
    MultiSetStateScenario(): CacheCoherenceScenario()
    {
        setup("MultiSetStateScenario",
            &CacheCoherenceScenario::init,
            &CacheCoherenceScenario::finish,
            &MultiSetStateScenario::getState);
    }

    virtual ~MultiSetStateScenario() {}

    virtual std::string getState() { return model.getMultiSetState(); }
};

class SetStateScenario: public CacheCoherenceScenario
{
public:
    SetStateScenario(): CacheCoherenceScenario()
    {
        setup("SetStateScenario",
            &CacheCoherenceScenario::init,
            &CacheCoherenceScenario::finish,
            &SetStateScenario::getState);
    }

    virtual ~SetStateScenario() {}

    virtual std::string getState() { return model.getSetState(); }
};

}}} // namespace cpptesk::examples::ccp

