/** @file fifo_scen.h
 * Header for scenarios of Multififo verification.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#pragma once

#include <multififo_media.h>

#include <ts/scenario.hpp>

using namespace cpptesk::ts;
using namespace cpptesk::hw;
//using namespace cpptesk::tracer;

namespace cpptesk {
namespace examples {
namespace fifo {

/// Common class for scenarios of testing of the DUT.
CPPTESK_SCENARIO(FIFOScenario)
{
public:
    // Type of iteration context
    typedef IntAbcCtx Ctx;

    /// Scenario initialization.
    virtual bool init(int argc, char ** argv);
    /// Finish scenario processing.
    virtual void finish();

    /// Scenario function NOP - give the control flow to the simulator to simulate 1 cycle
    bool scen_nop(Ctx& ctx);
    /// Scenario functions "Push the message into the buffers".
    bool scen_push_a(Ctx& ctx);
    bool scen_push_b(Ctx& ctx);
    /// Scenario function "Pop the message from the buffer".
    bool scen_pop(Ctx& ctx);

    /// Get current state. @return Filness of the buffers and ready statuses of ifaces.
    std::string get_state_fifo();

    FIFOScenario(void);
    virtual ~FIFOScenario(void);

protected:
    /// Object of reference model class.
    MultififoMediator dut;
};

/// Child-class from common scenario class.
class ExampleScenario : public FIFOScenario
{
public:
    ExampleScenario();
    virtual ~ExampleScenario();
};

}}}
