/*
 * Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Temporal sequence guard 
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

#include <hw/clone.hpp>
#include <hw/unused.hpp>
#include <hw/utils/range.hpp>
#include <hw/utils/shared_ptr.hpp>

namespace cpptesk {
namespace hw {
namespace sequence {

////////////////////////////////////////////////////////////////////////////////
// Guard
////////////////////////////////////////////////////////////////////////////////

/**
 * Guard waits until the event occures.
 */
class Guard
{
public:
    Guard(const Event &event): event(event) {}
    virtual ~Guard() {}

    const Event& getEvent() const { return event; }

protected:
    const Event &event;
};

/// Temporal operator 'wait_until' is usually used in the middle of a sequence.
/// Example: E1 << wait_until(E2) << delay(1) << E3;
inline Guard wait_until(const Event &event)
{
    return Guard(event);
}

/// Temporal operator 'if_then' is equivalent to 'wait_until', but  usually used in the beginning.
/// Example: if_then(E1) << any_delay() << E2;
inline Guard if_then(const Event &event)
{
    return wait_until(event);
}

/// Temporal operator '*' is a short form of 'wait_until'.
/// Example: E1 << *E2 << delay(1) << E3;
inline Guard operator * (const Event &event)
{
    return wait_until(event);
}

}}} // namespace cpptesk::hw::sequence

