/*
 * Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Event manager 
///////////////////////////////////////////////////////////////////////////////

#include <hw/sequence/manager.hpp>

namespace cpptesk {
namespace hw {
namespace sequence {

////////////////////////////////////////////////////////////////////////////////
// Model Event
////////////////////////////////////////////////////////////////////////////////

bool ModelEvent::occured() const
{
    return _manager.contains(*this);
}

/**
 * Sequence repetition.
 */
class Repeat: public Range<unsigned>
{
public:
    Repeat(): Range<unsigned>(1) {}
    Repeat(unsigned delay): Range<unsigned>(delay) {}
    Repeat(unsigned min, unsigned max): Range<unsigned>(min, max) {}

    bool isStar() const { return min == 0 && max == INFINITY; }
    bool isPlus() const { return min == 1 && max == INFINITY; }

    bool isStarOrPlus() const { return min <= 1 && max == INFINITY; }
};

}}} // namespace cpptesk::hw::sequence

