/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _TESTREG_H_INCLUDED_
#define _TESTREG_H_INCLUDED_

#ifdef __cplusplus
#include <netfsm/engines.hpp>
using namespace cpptesk::ts;
#else
#include <netfsm/engines.h>
#endif // __cplusplus

#define CPPTESK_TEST_REGISTRY_BEGIN \
		test_registry_item test_registry[] = {

#define CPPTESK_TEST_REGISTRY_END \
		{ NULL, CTESK, NULL, NULL, NULL } };

#define CPPTESK_REGISTER_CPPTESK_TEST(test, engine, params) \
		{ #test, CPPTESK, &test, engine, params },

#define CPPTESK_REGISTER_CTESK_TEST(test, engine, params) \
		{ #test, CTESK, &test, engine, params },

#define CPPTESK_REGISTER_TEST(test, engine, params) \
		CPPTESK_REGISTER_CPPTESK_TEST(test, engine, params)

#if (defined(CPPTESK_SHORT_NAMES) || defined(CPPTESK_SHORT_SHORT_NAMES))
#define TEST_REGISTRY_BEGIN                         CPPTESK_TEST_REGISTRY_BEGIN
#define TEST_REGISTRY_END                           CPPTESK_TEST_REGISTRY_END
#define REGISTER_CPPTESK_TEST(test, engine, params) CPPTESK_REGISTER_CPPTESK_TEST(test, engine, params)
#define REGISTER_CTESK_TEST(test, engine, params)   CPPTESK_REGISTER_CTESK_TEST(test, engine, params)
#define REGISTER_TEST(test, engine, params)         CPPTESK_REGISTER_TEST(test, engine, params)
#endif

typedef enum ScenarioType {CTESK, CPPTESK} ScenarioType;

#define SCEN_ARGV_LENGTH 1024

typedef struct test_registry_item {
    const char *testName;
    ScenarioType testType;
    void *testScenario;
    TestEngine *testEngine;
    const char *testParams;
} test_registry_item;

extern test_registry_item test_registry[];

#include <netfsm/agent.h>
#include <netfsm/generator.h>

typedef enum { LOCAL, NETWORK } run_type_t;

#ifdef __cplusplus
void set_scenario_wrapper(CppTestScenario *testScenario, TestEngine *testEngine, char *parameters, bool netscen);
void set_scenario_wrapper_with_init(CppTestScenario *initTestScenario, TestEngine *testEngine, char *parameters, bool netscen);
void scenario_wrapper_cpp(void);
void scenario_wrapper_with_init_cpp(void);
int startscen_cpp_TestRegistry(bool netscen, char *scen_name, char *scen_params);
int startScenario_TestRegistry(const char *name, run_type_t type);

// For compatibility with old test systems. (written before April 2011)
/*
int start_scenario(CppTestScenario *scen, char *params);
int start_net_scenario(CppTestScenario *scen, char *params);
int start_scenario_with_init(CppTestScenario *initscen, cpptesk::ts::CppTestScenario *scen, char *params);
int start_net_scenario_with_init(CppTestScenario *initscen, cpptesk::ts::CppTestScenario *scen, char *params);
int start_scenario_new(CppTestScenario *scen, TestEngine *testEngine, char *params, bool netscen);
int start_scenario_new_with_init(CppTestScenario *initscen, CppTestScenario *scen, TestEngine *testEngine, char *params, bool netscen);
*/
#endif

#ifdef __cplusplus
extern "C" {
#endif

void set_scenario_wrapper(TestScenario *testScenario, TestEngine *testEngine, char *parameters, bool netscen);
void set_scenario_wrapper_with_init(TestScenario *initTestScenario, TestScenario *testScenario, TestEngine *testEngine, char *parameters, bool netscen);
void scenario_wrapper(void);
void scenario_wrapper_with_init(void);
int find_scenario_TestRegistry(char *scen_name, char *scen_params);
int startscen_TestRegistry(bool netscen, char *scen_name, char *scen_params);
int startScenario_c_TestRegistry(const char *name, run_type_t type);

// For compatibility with old test systems. (written before April 2011)
/*
int start_scenario(TestScenario *scen, char *params);
int start_net_scenario(TestScenario *scen, char *params);
int start_scenario_with_init(TestScenario *initscen, TestScenario *scen, char *params);
int start_net_scenario_with_init(TestScenario *initscen, TestScenario *scen, char *params);
int start_scenario_new(TestScenario *scen, TestEngine *testEngine, char *params, bool netscen);
int start_scenario_new_with_init(TestScenario *initscen, TestScenario *scen, TestEngine *testEngine, char *params, bool netscen);
*/

#ifdef __cplusplus
}
#endif

#endif // _TESTREG_H_INCLUDED_

